@@ -19,10 +19,26 @@ class (Show a) => Pretty a where
     pretty :: a -> String
     pretty x = show x
 
+-- Unmatched right curly bracket at - line 1, at end of line
+-- syntax error at - line 1, near "}"
+-- Execution of - aborted due to compilation errors.
+
+instance Pretty VStr
+
+instance Pretty Exp where
+    pretty (Val (VError msg (NonTerm pos))) = "Syntax error at " ++ (show pos) ++ msg
+    pretty (Val v) = pretty v
+    pretty (Syn x vs) = "Syn " ++ pretty x ++ " {{ " ++ joinList "; " (map pretty vs) ++ " }}"
+    pretty x = show x
+
+instance Pretty Env where
+    pretty x = "{ " ++ (pretty $ envBody x) ++ " }"
+
 instance Pretty Val where
-    pretty (VJunc j l) = "(" ++ joinList mark items ++ ")"
+    pretty (VJunc (Junc j dups vals)) = "(" ++ joinList mark items ++ ")"
         where
-        items = map pretty $ setToList l
+        items = map pretty $ values
+        values = setToList vals ++ (concatMap (replicate 2)) (setToList dups)
         mark  = case j of
             JAny  -> " | "
             JAll  -> " & "
@@ -35,9 +51,15 @@ instance Pretty Val where
     pretty (VStr x) = show x -- XXX escaping
     pretty (VRat x) = show $ (fromIntegral $ numerator x) / (fromIntegral $ denominator x)
     pretty (VComplex x) = show x
-    pretty (VRef (VList x)) = "[" ++ joinList ", " (map pretty x) ++ "]"
+    pretty (VRef (VList x))
+        | not . null . (drop 100) $ x
+        = "[" ++ pretty (head x) ++ ", ...]"
+        | otherwise = "[" ++ joinList ", " (map pretty x) ++ "]"
     pretty (VRef x) = "\\(" ++ pretty x ++ ")"
-    pretty (VList x) = "(" ++ joinList ", " (map pretty x) ++ ")"
+    pretty (VList x)
+        | not . null . (drop 100) $ x
+        = "(" ++ pretty (head x) ++ ", ...)"
+        | otherwise = "(" ++ joinList ", " (map pretty x) ++ ")"
     pretty (VSub x) = "sub {...}"
     pretty (VBlock x) = "{...}"
     pretty (VError x y) = "*** Error: " ++ x ++ "\n    in " ++ show y
